import { all, delay, put, takeLatest } from 'redux-saga/effects'

import {
  CANCEL_ALL_AND_RESERVE_SEAT_START,
  CANCEL_SEAT_RESERVATION_START,
  GET_STATUSES_START,
  RESERVE_SEAT_START
} from '../actions/actionTypes'

import {
  getStatusesFail,
  getStatusesLoading,
  getStatusesSuccess
} from '../actions'

import {
  cancelSeatReservation,
  getStatuses,
  reserveSeat
} from '../../services/httpService'
import {
  cancelSeatReservationFail,
  cancelSeatReservationLoading,
  cancelSeatReservationSuccess,
  getStatusesStart as getStatusesActionStart,
  reserveSeatFail,
  reserveSeatLoading,
  reserveSeatSuccess
} from '../actions/seatMapActions'
import { notify } from '../actions/notificationsActions'
import { MESSAGES } from 'seat-map-view'

const resetReservationData = () => {
  localStorage.setItem('reservationData', JSON.stringify([]))
}

function* getStatusesStart({ payload: { delayTime = 0 } = {} }) {
  try {
    if (delayTime) {
      yield delay(delayTime)
    }
    yield put(getStatusesLoading())
    const result = yield getStatuses()
    const statuses = result.data.data.attributes
    const ownReservations = []
    Object.keys(statuses).forEach((groupRowId) =>
      Object.keys(statuses[groupRowId]).forEach((seatIndex) => {
        if (statuses[groupRowId][seatIndex] === 'OR') {
          ownReservations.push(`${groupRowId}:${seatIndex}`)
        } else if (statuses[groupRowId][seatIndex] === 'H') {
          statuses[groupRowId][seatIndex] = 'BLOCKED'
        }
      })
    )
    yield put(getStatusesSuccess(statuses))
    yield localStorage.setItem(
      'ownReservations',
      JSON.stringify(ownReservations)
    )
    if (!ownReservations.length) {
      yield resetReservationData()
    }
    if (ownReservations.length === 0) {
      yield localStorage.removeItem('tierId')
    }
    if(process.env.NODE_ENV === 'production') {
      yield put(getStatusesActionStart({ delayTime: 3000 }))
    }
  } catch (e) {
    //TODO try getting statuses 3 times
    yield put(getStatusesFail(e))
  }
}

function* reserveSeatStart({ payload }) {
  try {
    yield put(reserveSeatLoading())
    const result = yield reserveSeat(payload.tierId, payload.seatId)
    yield localStorage.setItem(
      'ownReservations',
      JSON.stringify(result.data.ownReservations)
    )
    if (!result.data.ownReservations.length) {
      yield resetReservationData()
    }
    const [groupRowId, seatIndex] = payload.seatId.split(':')
    yield put(reserveSeatSuccess({ groupRowId, seatIndex }))
    window.document.dispatchEvent(
      new CustomEvent('seatClick', {
        detail: {
          type: 'reserve',
          seatId: payload.seatId,
          tierId: payload.tierId
        }
      })
    )
  } catch (e) {
    if (!e.response || !e.response.status) {
      yield put(notify({ type: 'alert', message: MESSAGES.SESSION_EXPIRED }))
    } else {
      if (e.response.data.message === MESSAGES.ALREADY_RESERVED) {
        yield put(notify({ type: 'alert', message: MESSAGES.ALREADY_RESERVED }))
      } else if (
        e.response.data.message === 'Maximum ticket reservation count exceeded.'
      ) {
        yield put(notify({ type: 'alert', message: MESSAGES.LIMIT_EXCEEDED }))
      } else {
        yield put(notify({ type: 'alert', message: MESSAGES.UNEXPECTED_ERROR }))
      }
    }

    yield put(reserveSeatFail(e))
  }
}

function* cancelAllAndReserveSeatStart({ payload }) {
  try {
    const ownReservations = JSON.parse(localStorage.getItem('ownReservations'))
    const tierId = localStorage.getItem('tierId')

    yield put(cancelSeatReservationLoading())
    const cancelResult = yield cancelSeatReservation(
      parseInt(tierId),
      ownReservations
    )
    yield localStorage.setItem(
      'ownReservations',
      JSON.stringify(cancelResult.data.ownReservations)
    )
    if (!cancelResult.data.ownReservations.length) {
      yield resetReservationData()
    }
    const canceledSeats = ownReservations.map((seatId) => {
      const [groupRowId, seatIndex] = seatId.split(':')
      return { groupRowId, seatIndex }
    })
    yield put(cancelSeatReservationSuccess({ canceledSeats }))

    yield put(reserveSeatLoading())
    const result = yield reserveSeat(payload.tierId, payload.seatId)
    yield localStorage.setItem(
      'ownReservations',
      JSON.stringify(result.data.ownReservations)
    )
    if (!result.data.ownReservations.length) {
      yield resetReservationData()
    }
    const [groupRowId, seatIndex] = payload.seatId.split(':')
    yield put(reserveSeatSuccess({ groupRowId, seatIndex }))
    window.document.dispatchEvent(
      new CustomEvent('seatClick', {
        detail: {
          type: 'cancelAllAndReserve',
          seatId: payload.seatId,
          tierId: payload.tierId
        }
      })
    )
  } catch (e) {
    if (!e.response || !e.response.status) {
      yield put(notify({ type: 'alert', message: MESSAGES.SESSION_EXPIRED }))
    } else {
      if (e.response.data.message === MESSAGES.ALREADY_RESERVED) {
        yield put(notify({ type: 'alert', message: MESSAGES.ALREADY_RESERVED }))
      } else if (
        e.response.data.message === 'Maximum ticket reservation count exceeded.'
      ) {
        yield put(notify({ type: 'alert', message: MESSAGES.LIMIT_EXCEEDED }))
      } else {
        yield put(notify({ type: 'alert', message: MESSAGES.UNEXPECTED_ERROR }))
      }
    }

    yield put(reserveSeatFail(e))
  }
}

function* cancelSeatReservationStart({ payload }) {
  try {
    yield put(cancelSeatReservationLoading())
    const result = yield cancelSeatReservation(payload.tierId, payload.seatIds)
    yield localStorage.setItem(
      'ownReservations',
      JSON.stringify(result.data.ownReservations)
    )
    if (!result.data.ownReservations.length) {
      yield resetReservationData()
    }
    const canceledSeats = payload.seatIds.map((seatId) => {
      const [groupRowId, seatIndex] = seatId.split(':')
      return { groupRowId, seatIndex }
    })
    yield put(cancelSeatReservationSuccess({ canceledSeats }))
    window.document.dispatchEvent(
      new CustomEvent('seatClick', {
        detail: {
          type: 'cancel',
          seatIds: payload.seatIds,
          tierId: payload.tierId
        }
      })
    )
  } catch (e) {
    if (!e.response || !e.response.status) {
      yield put(notify({ type: 'alert', message: MESSAGES.SESSION_EXPIRED }))
    } else {
      yield put(notify({ type: 'alert', message: MESSAGES.UNEXPECTED_ERROR }))
    }

    yield put(cancelSeatReservationFail(e))
  }
}

function* seatMapSaga() {
  yield all([
    takeLatest(GET_STATUSES_START, getStatusesStart),
    takeLatest(RESERVE_SEAT_START, reserveSeatStart),
    takeLatest(CANCEL_SEAT_RESERVATION_START, cancelSeatReservationStart),
    takeLatest(CANCEL_ALL_AND_RESERVE_SEAT_START, cancelAllAndReserveSeatStart)
  ])
}

export { seatMapSaga }
